SUBROUTINE PL_debug(cd_space, cd_name, id_time, id_intdim, id_size, &
   ida_obj, id_err)
!
!**** TITLE - pl_debug : User defined debug procedures
!
!     Purpose:
!     --------
!     To apply user defined debug procedures to objects handled by PALM
!
!     Interface:
!     ----------
!     There is no direct call in the user's programs. This routine is 
!     activated by the PrePALM interface through the DEBUG parameter of
!     communications. If PL_DEBUG_ON_SEND is selected, the debug procedure
!     is applied during the execution of PALM_Put. If PL_DEBUG_ON_SEND 
!     is selected, the debug procedure is applied during the execution of 
!     PALM_Get. If PL_DEBUG_ON_BOTH is selected, the debug procedure is 
!     applied both during the execution on PALM_Put and the execution
!     of PALM_Get.
!     The user has to define his own debug procedures in the internal SELECT
!     CASE.
!     The variable id_size contains the size (number of elements) of the 
!     object to debug.
!     The error code id_err id user defined and it is returned as error 
!     code of PALM_Put or PALM_Get if no other major errors occur.
!
!***  Method:
!     -------
!     Select an appropriate debug procedure following the space name.
!     "Cast" the PALM object into the mould of the type of the concerned
!     space.
!     Apply the debug procedure.
!     Return a user defined error code.
!
!     Externals:
!     ----------
!     None
!
!     Files:
!     ------
!     None
!
!     References:
!     -----------
!     The PALM Software User's Guide
!
!     History:
!     --------
!      Version    Programmer      Date         Description
!      -------    ----------      ----         -----------
!       1.0       Andrea        1999/12/20     Complete reshape
!*--------------------------------------------------------------------------
!
!**   0. DECLARATIONS  
!        ------------
!
!**   0.1 Include files and modules
!
  USE palmlib     !*I The PALM interface
  USE palm_user_param
!
!**   0.2 Local variables 
!
  IMPLICIT NONE
!
!*V Palm Related Variables. DO NOT CHANGE IT.
!
  CHARACTER(LEN=PL_LNAME), INTENT(in)  :: cd_space  
  CHARACTER(LEN=PL_LNAME), INTENT(in)  :: cd_name
  INTEGER, INTENT(in)            :: id_time
  INTEGER, INTENT(in)            :: id_intdim
  INTEGER, INTENT(in)            :: id_size
  INTEGER, DIMENSION(id_intdim), INTENT(in) :: ida_obj
  INTEGER, INTENT(out)           :: id_err
  REAL :: rd_J0
  REAL, DIMENSION(ip_dimobs) :: rda_Hx
  REAL, DIMENSION(ip_dimx) :: rda_x
!
!*V Local user defined variables
!
!  FOR EXAMPLE
!
!  REAL(KIND=8), DIMENSION(id_size) :: dla_double
!
!  TYPE tder
!    SEQUENCE
!    INTEGER :: il_first
!    INTEGER :: il_second
!    REAL, DIMENSION(12) :: rla_real
!  END TYPE tder
!  TYPE (tder), DIMENSION(id_size) :: sla_type
!
!  INTEGER :: ib
!
!*--------------------------------------------------------------------------
!
!**   1. DEBUG PROCEDURES
!        ----------------
!
!**   1.1 Entering the selection : DO NOT CHANGE THIS LINES 
!
  id_err = 0 
  WRITE(PL_OUT,*) 'DEBUG: Applying debug to object ',cd_name 
  WRITE(PL_OUT,*) '       at time ', id_time, 'for space ',cd_space
  SELECT CASE(cd_space)
!
!**   1.2 User defined cases
!
! FOR EXAMPLE
!
!!*C Remember than PALM identifies spaces by a string composed by
!!   the space name suffixed by the name of the unit defining the
!!   the space. For instance, if the space "dspace" is defined in
!!   the identity card of the unit "the_unit" the corresponding
!!   internal PALM identifier is "dspace.the_unit"
!
!  CASE('dspace.the_unit')
!!
!!*C The following line has to start each case block.
!!   The format is user_variable = TRANSFER(ida_obj, user_variable)
!!   Where user_variable declaration corresponds to the type 
!!   associated to the space
!
!      dla_double = TRANSFER(ida_obj, dla_double)
!
!!*C In the case of multidimensional arrays this line should
!!   take the form
!!
!!      dla_double = RESHAPE(TRANSFER(ida_obj, dla_double), &
!                           SHAPE(dla_double))
!
!      WRITE(PL_OUT,*) 'Min Value = ', MINVAL(dla_double)
!      WRITE(PL_OUT,*) 'Max Value = ', MAXVAL(dla_double)
!      IF (MAXVAL(dla_double) .gt. 40.d0) THEN
!         WRITE(PL_OUT,*) 'ERROR : Value of ',cd_name,' is too high'
!         id_err = 999
!      END IF
!      DO ib = 1,id_size
!	WRITE(PL_OUT,*) dla_double(ib)
!      END DO
!  CASE('tspace.its_unit')
!      sla_type = TRANSFER(ida_obj, sla_type)
!      DO ib = 1,id_size
!	 WRITE(PL_OUT,*) sla_type(ib)%il_first
!	 WRITE(PL_OUT,*) sla_type(ib)%il_second
!        WRITE(PL_OUT,*) 'Min Value = ', MINVAL(sla_type(ib)%rla_real)
!        WRITE(PL_OUT,*) 'Max Value = ', MAXVAL(sla_type(ib)%rla_real)
!      END DO
!
!**   1.3 End of the selection. DO NOT CHANGE THIS LINES.
!
  CASE ('vectobs.INV_R','vectobs.H','vectobs.Hstar','vectobs.H_LT','vectobs.WRTOBS','vectobs.LECTOBS','vectobs.INIT_LECTOBS')
      rda_Hx = TRANSFER(ida_obj, rda_Hx)
      WRITE(PL_OUT,*) 'Min Value in Hx = ', MINVAL(rda_Hx)
      WRITE(PL_OUT,*) 'Max Value in Hx = ', MAXVAL(rda_Hx)
      WRITE(PL_OUT,*) '2 Norm of Hx    = ', SQRT(DOT_PRODUCT(rda_Hx,rda_Hx))
  CASE ('crt_space.H','crt_space.H_LT','crt_space.MODEL','crt_space.sqrtB',&
      'crt_space.sqrtB_T','crt_space.Hstar',&
      'crt_space.INITv')
      rda_x = TRANSFER(ida_obj, rda_x)
      WRITE(PL_OUT,*) 'Min Value in crt = ', MINVAL(rda_x)
      WRITE(PL_OUT,*) 'Max Value in crt = ', MAXVAL(rda_x)
      WRITE(PL_OUT,*) '2 Norm of crt    = ', SQRT(DOT_PRODUCT(rda_x,rda_x))
  CASE ('one_real','one_real.WRTOBS','one_real.LECTOBS')
      rd_J0 = TRANSFER(ida_obj,rd_J0)
      WRITE(PL_OUT,*) 'one real = ', rd_J0
  CASE default
      WRITE(PL_OUT,*) 'DEBUG: No debug action associated '
      WRITE(PL_OUT,*) '       to space ',cd_space
  END SELECT
  CALL flush(PL_OUT)
!	  
!*--------------------------------------------------------------------------
!
END SUBROUTINE pl_debug
!	  
!*==========================================================================


